@@ -262,26 +262,6 @@ mai_atk_object_get_type(void)
     return type;
 }
 
-/*
- * Must keep sychronization with enumerate AtkProperty in 
- * accessible/src/base/nsAccessibleEventData.h
- */
-static char * sAtkPropertyNameArray[PROP_LAST] = {
-    0,
-    "accessible-name",
-    "accessible-description",
-    "accessible-parent",
-    "accessible-role",
-    "accessible-layer",
-    "accessible-mdi-zorder",
-    "accessible-table-caption",
-    "accessible-table-column-description",
-    "accessible-table-column-header",
-    "accessible-table-row-description",
-    "accessible-table-row-header",
-    "accessible-table-summary"
-};
-
 #ifdef MAI_LOGGING
 PRInt32 nsAccessibleWrap::mAccWrapCreated = 0;
 PRInt32 nsAccessibleWrap::mAccWrapDeleted = 0;
@@ -777,7 +757,7 @@ getRoleCB(AtkObject *aAtkObj)
 
     if (aAtkObj->role == ATK_ROLE_INVALID) {
         PRUint32 accRole, atkRole;
-        nsresult rv = accWrap->GetFinalRole(&accRole);
+        nsresult rv = accWrap->GetRole(&accRole);
         NS_ENSURE_SUCCESS(rv, ATK_ROLE_INVALID);
 
         atkRole = atkRoleMap[accRole]; // map to the actual value
@@ -1053,7 +1033,6 @@ refRelationSetCB(AtkObject *aAtkObj)
         return relation_set;
     }
 
-    AtkObject *accessible_array[1];
     AtkRelation* relation;
     
     PRUint32 relationType[] = {nsIAccessibleRelation::RELATION_LABELLED_BY,
@@ -1074,14 +1053,28 @@ refRelationSetCB(AtkObject *aAtkObj)
             atk_relation_set_remove(relation_set, relation);
         }
 
-        nsIAccessible* accRelated;
-        nsresult rv = accWrap->GetAccessibleRelated(relationType[i], &accRelated);
-        if (NS_SUCCEEDED(rv) && accRelated) {
-            accessible_array[0] = nsAccessibleWrap::GetAtkObject(accRelated);
-            relation = atk_relation_new(accessible_array, 1,
-                                        static_cast<AtkRelationType>(relationType[i]));
-            atk_relation_set_add(relation_set, relation);
-            g_object_unref(relation);
+        nsCOMPtr<nsIAccessibleRelation> geckoRelation;
+        nsresult rv = accWrap->GetRelationByType(relationType[i],
+                                                 getter_AddRefs(geckoRelation));
+        if (NS_SUCCEEDED(rv) && geckoRelation) {
+            PRUint32 targetsCount = 0;
+            geckoRelation->GetTargetsCount(&targetsCount);
+            if (targetsCount) {
+                AtkObject** accessible_array = new AtkObject*[targetsCount];
+                for (PRUint32 index = 0; index < targetsCount; index++) {
+                    nsCOMPtr<nsIAccessible> geckoTarget;
+                    geckoRelation->GetTarget(index, getter_AddRefs(geckoTarget));
+                    accessible_array[index] =
+                        nsAccessibleWrap::GetAtkObject(geckoTarget);
+                }
+
+                relation = atk_relation_new(accessible_array, targetsCount,
+                                            static_cast<AtkRelationType>(relationType[i]));
+                atk_relation_set_add(relation_set, relation);
+                g_object_unref(relation);
+
+                delete [] accessible_array;
+            }
         }
     }
 
@@ -1113,7 +1106,7 @@ nsAccessibleWrap *GetAccessibleWrap(AtkObject *aAtkObj)
     return tmpAccWrap;
 }
 
-NS_IMETHODIMP
+nsresult
 nsAccessibleWrap::FireAccessibleEvent(nsIAccessibleEvent *aEvent)
 {
     nsresult rv = nsAccessible::FireAccessibleEvent(aEvent);
