@@ -40,6 +40,7 @@
 #define nsEventStateManager_h__
 
 #include "nsIEventStateManager.h"
+#include "nsEvent.h"
 #include "nsGUIEvent.h"
 #include "nsIContent.h"
 #include "nsIObserver.h"
@@ -58,14 +59,13 @@ class nsIPresShell;
 class nsIDocShell;
 class nsIDocShellTreeNode;
 class nsIDocShellTreeItem;
-class nsIFocusController;
 class imgIContainer;
 class nsDOMDataTransfer;
 
 // mac uses click-hold context menus, a holdover from 4.x
-// touch screens (like hildon) could use this also, 
+// touch screens (like maemo) could use this also, 
 // perhaps we should move to NS_TOUCHSCREEN
-#if defined(XP_MACOSX) || defined(MOZ_PLATFORM_HILDON)
+#if defined(XP_MACOSX) || defined(MOZ_PLATFORM_MAEMO)
 #define CLICK_HOLD_CONTEXT_MENUS 1
 #endif
 
@@ -78,6 +78,7 @@ class nsEventStateManager : public nsSupportsWeakReference,
                             public nsIEventStateManager,
                             public nsIObserver
 {
+  friend class nsMouseWheelTransaction;
 public:
   nsEventStateManager();
   virtual ~nsEventStateManager();
@@ -121,11 +122,7 @@ public:
 
   NS_IMETHOD GetContentState(nsIContent *aContent, PRInt32& aState);
   virtual PRBool SetContentState(nsIContent *aContent, PRInt32 aState);
-  NS_IMETHOD GetFocusedContent(nsIContent **aContent);
-  NS_IMETHOD SetFocusedContent(nsIContent* aContent);
-  NS_IMETHOD GetLastFocusedContent(nsIContent **aContent);
-  NS_IMETHOD GetFocusedFrame(nsIFrame **aFrame);
-  NS_IMETHOD ContentRemoved(nsIContent* aContent);
+  NS_IMETHOD ContentRemoved(nsIDocument* aDocument, nsIContent* aContent);
   NS_IMETHOD EventStatusOK(nsGUIEvent* aEvent, PRBool *aOK);
 
   // Access Key Registration
@@ -137,15 +134,6 @@ public:
                        PRBool aHaveHotspot, float aHotspotX, float aHotspotY,
                        nsIWidget* aWidget, PRBool aLockCursor);
 
-  NS_IMETHOD ShiftFocus(PRBool aForward, nsIContent* aStart=nsnull);
-
-  virtual PRBool GetBrowseWithCaret();
-  void ResetBrowseWithCaret();
-
-  NS_IMETHOD MoveFocusToCaret(PRBool aCanFocusDoc, PRBool *aIsSelectionWithFocus);
-  NS_IMETHOD MoveCaretToFocus();
-  NS_IMETHOD ChangeFocusWith(nsIContent* aFocus, EFocusedWithType aFocusedWith);
-
   static void StartHandlingUserInput()
   {
     ++sUserInputEventDepth;
@@ -167,14 +155,6 @@ public:
                                            nsIEventStateManager)
 
 protected:
-  /**
-   * In certain situations the focus controller's concept of focus gets out of
-   * whack with mCurrentFocus. This is used in known cases to reset the focus
-   * controller's focus. At some point we should probably move to a single
-   * focus storage mechanism because tracking it in several places is error-prone.
-   */
-  void EnsureFocusSynchronization();
-
   void UpdateCursor(nsPresContext* aPresContext, nsEvent* aEvent, nsIFrame* aTargetFrame, nsEventStatus* aStatus);
   /**
    * Turn a GUI mouse event into a mouse event targeted at the specified
@@ -221,20 +201,8 @@ protected:
                            nsWeakFrame& aTargetFrame);
   nsresult SetClickCount(nsPresContext* aPresContext, nsMouseEvent *aEvent, nsEventStatus* aStatus);
   nsresult CheckForAndDispatchClick(nsPresContext* aPresContext, nsMouseEvent *aEvent, nsEventStatus* aStatus);
-  nsresult GetNextTabbableContent(nsIContent* aRootContent,
-                                  nsIContent* aStartContent,
-                                  nsIFrame* aStartFrame,
-                                  PRBool forward, PRBool ignoreTabIndex,
-                                  nsIContent** aResultNode,
-                                  nsIFrame** aResultFrame);
-  nsIContent *GetNextTabbableMapArea(PRBool aForward, nsIContent *imageContent);
-
-  PRInt32 GetNextTabIndex(nsIContent* aParent, PRBool foward);
-  nsresult SendFocusBlur(nsPresContext* aPresContext, nsIContent *aContent, PRBool aEnsureWindowHasFocus);
-  void EnsureDocument(nsIPresShell* aPresShell);
   void EnsureDocument(nsPresContext* aPresContext);
   void FlushPendingEvents(nsPresContext* aPresContext);
-  nsIFocusController* GetFocusControllerForDocument(nsIDocument* aDocument);
 
   /**
    * The phases of HandleAccessKey processing. See below.
@@ -257,7 +225,7 @@ protected:
    * @param aStatus the event status
    * @param aBubbledFrom is used by an ancestor to avoid calling HandleAccessKey()
    *        on the child the call originally came from, i.e. this is the child
-   *        that recursively called us in it's Up phase. The initial caller
+   *        that recursively called us in its Up phase. The initial caller
    *        passes |nsnull| here. This is to avoid an infinite loop.
    * @param aAccessKeyState Normal, Down or Up processing phase (see enums
    *        above). The initial event receiver uses 'normal', then 'down' when
@@ -278,18 +246,8 @@ protected:
   // DocShell Focus Traversal Methods
   //---------------------------------------------
 
-  nsresult ShiftFocusInternal(PRBool aForward, nsIContent* aStart = nsnull);
-  void TabIntoDocument(nsIDocShell* aDocShell, PRBool aForward);
-  void ShiftFocusByDoc(PRBool forward);
-  PRBool IsFrameSetDoc(nsIDocShell* aDocShell);
-  PRBool IsIFrameDoc(nsIDocShell* aDocShell);
+  nsIContent* GetFocusedContent();
   PRBool IsShellVisible(nsIDocShell* aShell);
-  void GetLastChildDocShell(nsIDocShellTreeItem* aItem,
-                            nsIDocShellTreeItem** aResult);
-  void GetNextDocShell(nsIDocShellTreeNode* aNode,
-                       nsIDocShellTreeItem** aResult);
-  void GetPrevDocShell(nsIDocShellTreeNode* aNode,
-                       nsIDocShellTreeItem** aResult);
 
   // These functions are for mousewheel and pixel scrolling
   nsresult GetParentScrollingView(nsInputEvent* aEvent,
@@ -312,10 +270,9 @@ protected:
   } ScrollQuantity;
   nsresult DoScrollText(nsPresContext* aPresContext,
                         nsIFrame* aTargetFrame,
-                        nsInputEvent* aEvent,
-                        PRInt32 aNumLines,
-                        PRBool aScrollHorizontal,
-                        ScrollQuantity aScrollQuantity);
+                        nsMouseScrollEvent* aMouseEvent,
+                        ScrollQuantity aScrollQuantity,
+                        PRBool aAllowScrollSpeedOverride);
   void DoScrollHistory(PRInt32 direction);
   void DoScrollZoom(nsIFrame *aTargetFrame, PRInt32 adjustment);
   nsresult GetMarkupDocumentViewer(nsIMarkupDocumentViewer** aMv);
@@ -323,6 +280,16 @@ protected:
   nsresult ChangeFullZoom(PRInt32 change);
   // end mousewheel functions
 
+  /*
+   * When a touch gesture is about to start, this function determines what
+   * kind of gesture interaction we will want to use, based on what is
+   * underneath the initial touch point.
+   * Currently it decides between panning (finger scrolling) or dragging
+   * the target element, as well as the orientation to trigger panning and
+   * display visual boundary feedback. The decision is stored back in aEvent.
+   */
+  void DecideGestureEvent(nsGestureNotifyEvent* aEvent, nsIFrame* targetFrame);
+
   // routines for the d&d gesture tracking state machine
   void BeginTrackingDragGesture ( nsPresContext* aPresContext, nsMouseEvent* inDownEvent,
                                   nsIFrame* inDownFrame ) ;
@@ -371,15 +338,7 @@ protected:
    */
   void FillInEventFromGestureDown(nsMouseEvent* aEvent);
 
-  PRBool mSuppressFocusChange; // Used only for Ender text fields to suppress a focus firing on mouse down
-
-  nsresult SetCaretEnabled(nsIPresShell *aPresShell, PRBool aVisibility);
-  nsresult SetContentCaretVisible(nsIPresShell* aPresShell, nsIContent *aContent, PRBool aVisible);
-  void FocusElementButNotDocument(nsIContent *aElement);
-
-  // Return the location of the caret
-  nsresult GetDocSelectionLocation(nsIContent **start, nsIContent **end, 
-                                   nsIFrame **startFrame, PRUint32 *startOffset);
+  nsresult DoContentCommandEvent(nsContentCommandEvent* aEvent);
 
   PRInt32     mLockCursor;
 
@@ -390,7 +349,7 @@ protected:
   nsWeakFrame mLastDragOverFrame;
 
   // member variables for the d&d gesture state machine
-  nsPoint mGestureDownPoint; // screen coordinates
+  nsIntPoint mGestureDownPoint; // screen coordinates
   // The content to use as target if we start a d&d (what we drag).
   nsCOMPtr<nsIContent> mGestureDownContent;
   // The content of the frame where the mouse-down event occurred. It's the same
@@ -411,20 +370,6 @@ protected:
   nsCOMPtr<nsIContent> mHoverContent;
   nsCOMPtr<nsIContent> mDragOverContent;
   nsCOMPtr<nsIContent> mURLTargetContent;
-  nsCOMPtr<nsIContent> mCurrentFocus;
-  nsCOMPtr<nsIContent> mLastFocus;
-  nsWeakFrame mCurrentFocusFrame;
-  PRInt32 mCurrentTabIndex;
-  EFocusedWithType mLastFocusedWith;
-
-  // DocShell Traversal Data Memebers
-  nsCOMPtr<nsIContent> mLastContentFocus;
-
-  //Anti-recursive stack controls
-
-  nsCOMPtr<nsIContent> mFirstBlurEvent;
-  nsCOMPtr<nsIDocument> mFirstDocumentBlurEvent;
-  nsCOMPtr<nsIContent> mFirstFocusEvent;
 
   // The last element on which we fired a mouseover event, or null if
   // the last mouseover event we fired has finished processing.
@@ -445,17 +390,9 @@ protected:
 
   PRPackedBool m_haveShutdown;
 
-  // So we don't have to keep checking accessibility.browsewithcaret pref
-  PRPackedBool mBrowseWithCaret;
-
-  // Recursion guard for tabbing
-  PRPackedBool mTabbedThroughDocument;
-
   // Array for accesskey support
   nsCOMArray<nsIContent> mAccessKeys;
 
-  nsCOMArray<nsIDocShell> mTabbingFromDocShells;
-
   // Unlocks pixel scrolling
   PRPackedBool mLastLineScrollConsumedX;
   PRPackedBool mLastLineScrollConsumedY;
