@@ -20,7 +20,7 @@
  * Portions created by the Initial Developer are Copyright (C) 2006
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Jesse Ruderman
+ * Contributor(s): Igor Bukanov
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -36,24 +36,33 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-gTestfile = 'regress-327691-01.js';
+gTestfile = 'regress-354145-03.js';
 
-var summary = "Do not crash in js_IsXMLName";
-var BUGNUMBER = 327691;
+var BUGNUMBER = 354145;
+var summary = 'Immutable XML';
 var actual = 'No Crash';
 var expect = 'No Crash';
 
 printBugNumber(BUGNUMBER);
 START(summary);
 
-var A=<y/>;
-var B=A.h;
-var D=B.h;
-B.h=D.h;
-B[0]=B.h;
+var list = <></>
+var N = 10;
+for (var i = 0; i != N; ++i)
+  list[i] = <{"a"+i}/>;
 
-B.h=D.h; // Crashes at js_IsXMLName.
+function prepare()
+{
+  for (var i = N - 1; i >= 0; --i)
+    delete list[i];
+  gc();
+  return "test";
+}
 
+print("Before list.contains");
+list.contains({ toString: prepare });
+print("After list.contains");
 TEST(1, expect, actual);
+print("After TEST");
 
 END();
