@@ -15,12 +15,11 @@
  * The Original Code is JavaScript Engine testing utilities.
  *
  * The Initial Developer of the Original Code is
- * Netscape Communications Corp.
- * Portions created by the Initial Developer are Copyright (C) 2002
+ * Mozilla Foundation.
+ * Portions created by the Initial Developer are Copyright (C) 2006
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
- *   pschwartau@netscape.com
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -36,39 +35,42 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-/*
- *
- * Date:    16 Mar 2002
- * SUMMARY: Shouldn't crash if define |var arguments| inside a function
- * See http://bugzilla.mozilla.org/show_bug.cgi?id=131510
- *
- */
+var gTestfile = 'regress-360969-05.js';
 //-----------------------------------------------------------------------------
-var gTestfile = 'regress-131510-001.js';
-var BUGNUMBER = 131510;
-var summary = "Shouldn't crash if define |var arguments| inside a function";
-printBugNumber(BUGNUMBER);
-printStatus(summary);
+var BUGNUMBER = 360969;
+var summary = '2^17: local function';
+var actual = 'No Crash';
+var expect = 'No Crash';
 
+var global = this;
 
-function f() {var arguments;}
-f();
+//-----------------------------------------------------------------------------
+test();
 
+//-----------------------------------------------------------------------------
 
-/*
- * Put same example in function scope instead of global scope
- */
-function g() { function f() {var arguments;}; f();};
-g();
+function test()
+{
+  enterFunc ('test');
+  printBugNumber(BUGNUMBER);
+  printStatus (summary);
+ 
+  var start = new Date();
+  var p;
+  var i;
+  var limit = 2 << 16;
 
+  for (var i = 0; i < limit; i++)
+  {
+    eval('function pf' + i + '() {}');
+  }
 
-/*
- * Put these examples in eval scope
- */
-var s = 'function f() {var arguments;}; f();';
-eval(s);
+  reportCompare(expect, actual, summary);
 
-s = 'function g() { function f() {var arguments;}; f();}; g();';
-eval(s);
+  var stop = new Date();
+
+  print('Elapsed time: ' + Math.floor((stop - start)/1000) + ' seconds');
+
+  exitFunc ('test');
+}
 
-reportCompare('No Crash', 'No Crash', '');
