@@ -16,7 +16,7 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2008
+ * Portions created by the Initial Developer are Copyright (C) 2007
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s): Jesse Ruderman
@@ -35,13 +35,13 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-369696-01.js';
-//-----------------------------------------------------------------------------
-var BUGNUMBER = 396696;
-var summary = 'Do not assert: map->depth > 0" in js_LeaveSharpObject';
-var actual = '';
-var expect = '';
+var gTestfile = 'regress-385393-01.js';
 
+//-----------------------------------------------------------------------------
+var BUGNUMBER = 385393;
+var summary = 'Regression test for bug 385393';
+var actual = 'No Crash';
+var expect = 'No Crash';
 
 //-----------------------------------------------------------------------------
 test();
@@ -53,10 +53,13 @@ function test()
   printBugNumber(BUGNUMBER);
   printStatus (summary);
 
-  q = [];
-  q.__defineGetter__("0", q.toString);
-  q[2] = q;
-  q.toSource();
+  try
+  {
+    [].map(1 for (x in []));
+  }
+  catch(ex)
+  {
+  }
 
   reportCompare(expect, actual, summary);
 
