@@ -19,7 +19,7 @@
  * Portions created by the Initial Developer are Copyright (C) 2008
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Gary Kwong
+ * Contributor(s):
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -35,14 +35,13 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-452498-168-1.js';
+var gTestfile = 'regress-452498-110.js';
 //-----------------------------------------------------------------------------
 var BUGNUMBER = 452498;
 var summary = 'TM: upvar2 regression tests';
 var actual = '';
 var expect = '';
 
-//-------  Comment #168  From  Gary Kwong
 
 //-----------------------------------------------------------------------------
 test();
@@ -54,13 +53,30 @@ function test()
   printBugNumber(BUGNUMBER);
   printStatus (summary);
 
-// Assertion failure: (fun->u.i.script)->upvarsOffset != 0, at ../jsfun.cpp:1543
+// ------- Comment #110 From Brendan Eich
 
-  (
-    new Function("const x = (function () { if (1e+81){} else{x} } )"
-      ))();
+// (In reply to comment #107)
 
-  reportCompare(expect, actual, summary);
+  function f(a) { const b = a; print(++b); return b; }
+
+  expect = 'function f(a) { const b = a; print(+ b + 1); return b; }';
+  actual = f + '';
+  compareSource(expect, actual, 'function f(a) { const b = a; print(++b); return b; }');
+
+  expect = '01';
+  actual = 0;
+
+  function g(a) { const b = a; print(actual = ++b); return b; }
+  actual = String(actual) + g(1);
+  reportCompare(expect, actual, 'function g(a) { const b = a; print(actual = ++b); return b; }');
+
+  expect = '21';
+  actual = 0;
+
+  const x = 1; print(actual = ++x);
+  actual = String(actual) + x;
+
+  reportCompare(expect, actual, 'const x = 1; print(actual = ++x); ');
 
   exitFunc ('test');
 }
